--------------------------------------------------------------------------------
-- Copyright (c) 2002 by Trenz Electronic GmbH.
-- Brendel 20, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2Se Spartan-IIe Development System.
-- File:        cpld.vhd
-- Description: shows access of CPLD registers for TE-XC2Se.
-- History:     2002may8, TT, created.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use ieee.numeric_std.all;

entity cpld is
	Port (	clk48	: in  std_logic;						-- master clock
			rst		: in  std_logic;						-- master reset
			cpld_rw	: out std_logic;						-- read / write for cpld
			cpld_cs	: out std_logic;						-- chip select for cpld
			a		: out std_logic_vector(2 downto 1);		-- cpld register adress
			din		: in  std_logic_vector(7 downto 0);		-- cpld data bus
			dout	: out std_logic_vector(7 downto 0);		-- cpld data bus
			we		: out std_logic;						-- display ram write enable
			wclk	: out std_logic;						-- display ram write clock
			lcdout	: out std_logic_vector(7 downto 0);		-- display ram data
			aout	: out std_logic_vector(4 downto 0);		-- display ram address
			rs232in	: in  std_logic;
			rs232out: out std_logic
	);
end cpld;

architecture Behavioral of cpld is
	constant line1: string(1 to 16):= "Happy Developing";
	constant line2: string(1 to 16):= "Dip SW:         ";

	signal state	: std_logic_vector(4 downto 0); 		-- state counter fpr sequenzer
    signal lled		: std_logic_vector(7 downto 0);			-- local led
    signal lpb		: std_logic_vector(7 downto 0);			-- local pushbutton
	signal dipswitch: std_logic_vector(7 downto 0);			-- Dipswitch value

begin
	
	process(clk48, rst)
	begin
		if rst= '1' then
			state<= (others=> '0');
		elsif rising_edge(clk48) then
			state<= state + 1;								-- advance state on every clock
		end if;
	end process;


	process(clk48, rst)
	begin
		if rst= '1' then
			cpld_cs <= '1';
		elsif rising_edge(clk48) then
			case state is
				when "00000"	=>	a	<= "00";			-- address = 00 => push buttons
									cpld_cs <= '1';
									cpld_rw <= '1';			-- read from cpld
				when "00001"  =>	cpld_cs <= '0';			-- chip select for cpld active
				when "00010"  =>	lpb <= din;				-- latch cpld data
				when "00011"  =>	cpld_cs <= '1';			-- chip select inactive
				when "00100"  =>	a <= "10";				-- address = 01 => led's
									cpld_rw <= '0';			-- write to cpld	
				when "00101"  =>	dout <= lled;			-- put data on bus
				when "00110"  =>	cpld_cs <= '0';			-- generate chip select pulse
				when "00111"  =>	cpld_cs <= '1';
				when "01000"  =>	a <= "01";				-- address = 01 => Dip switch
									cpld_cs <= '1';
									cpld_rw <= '1';			-- read from cpld
				when "01001"  =>	cpld_cs <= '0';			-- chip select for cpld active
				when "01010"  =>	dipswitch <= din;		-- latch cpld data
				when "01011"  =>	cpld_cs <= '1';			-- chip select inactive
				when others  =>		cpld_cs <= '1';
			end case;

			lled(7 downto 1)  <= lpb(7 downto 1);			-- map pushbottons to LED
			if lpb(0) = '0' then							-- if PB1 not pressed
				rs232out <= rs232in;						-- loop RS232 signals
				lled(0) <=  not rs232in;						-- map RS232IN to L1
			else
				rs232out <= '1';						
				lled(0) <=  '1';
			end if;
		end if;
	end process;


	
	we <= '1';												-- write enable for display ram always active
	wclk <= not clk48;										-- output write clock for display ram
	aout <= state;											-- adress for ram ist state value
	
	process(clk48, rst)
	begin
		if rst= '1' then
			null;
		elsif rising_edge(clk48) then
			case state is
				-- write first line to display ram
				when "11111"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(1)), 8));
				when "00000"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(2)), 8));
				when "00001"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(3)), 8));
				when "00010"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(4)), 8));
				when "00011"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(5)), 8));
				when "00100"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(6)), 8));
				when "00101"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(7)), 8));
				when "00110"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(8)), 8));
				when "00111"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(9)), 8));
				when "01000"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(10)), 8));
				when "01001"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(11)), 8));
				when "01010"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(12)), 8));
				when "01011"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(13)), 8));
				when "01100"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(14)), 8));
				when "01101"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(15)), 8));
				when "01110"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line1(16)), 8));
				-- write second line to display ram
				when "01111"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(1)), 8));
				when "10000"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(2)), 8));
				when "10001"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(3)), 8));
				when "10010"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(4)), 8));
				when "10011"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(5)), 8));
				when "10100"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(6)), 8));
				when "10101"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(7)), 8));
				when "10110"  => lcdout <= std_logic_vector(to_unsigned(character'pos(line2(8)), 8));
				-- write dip switch data to display ram	  
				when "10111"  => lcdout <= "0011000" & not dipswitch(0); 
				when "11000"  => lcdout <= "0011000" & not dipswitch(1);
				when "11001"  => lcdout <= "0011000" & not dipswitch(2);
				when "11010"  => lcdout <= "0011000" & not dipswitch(3);
				when "11011"  => lcdout <= "0011000" & not dipswitch(4);
				when "11100"  => lcdout <= "0011000" & not dipswitch(5);
				when "11101"  => lcdout <= "0011000" & not dipswitch(6);
				when "11110"  => lcdout <= "0011000" & not dipswitch(7);
					  
				when others  =>  lcdout <= x"20";				-- blank
			end case;
		end if;
	end process;

	
	
end Behavioral;
