--------------------------------------------------------------------------------
-- Copyright (c) 2002 by Trenz Electronic GmbH.
-- Brendel 20, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2Se Spartan-IIe Development System.
-- Description: Demo LCD on XC2Se
-- History:     2002feb14, FB, created.
--				2002jun13, TT, modified
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity lcd is
	port (
		clk48:    in  std_logic;							-- master clock
		rst:      in  std_logic;							-- master reset
		ext_a:    out std_logic_vector(2 downto 0);			-- LCD/Expansion address
		ext_d:    out std_logic_vector(7 downto 0);			-- LCD/Expansion data
		ext_rw:   out std_logic;							-- LCD/Expansion read/write
		cs_lcd:   out std_logic;							-- LCD chip select active high
		csn_lcd:  out std_logic;							-- LCD chip select active low
		ram_adr:  out std_logic_vector(4 downto 0);			-- content input address
		ram_data: in  std_logic_vector(7 downto 0)			-- content input data
	);
end lcd;

--------------------------------------------------------------------------------
architecture bhv of lcd is
	signal clk: std_logic;									-- local clock
	signal div: unsigned (14 downto 0); 					-- local clock divider 
	signal reset: std_logic;								-- lcd reset sequence active

	signal cmd: std_logic_vector (7 downto 0);				-- byte written to lcd
	signal rs: std_logic;									-- register select bit for lcd
	signal idx: integer range 0 to 3;						-- sequencer index for lcd timing
	signal cs:  std_logic;									-- chip select for lcd
	signal cnt: unsigned (5 downto 0);						-- character, command counter
	
begin
	------------------------------------
	-- clock & reset
	process(clk48, rst)
	begin
		if rst= '1' then
			div <= (others=> '0');
		elsif rising_edge(clk48) then
			div<= div + 1;
		end if;
	end process;

	clk<= div(div'left);

	------------------------------------
	-- sequencer
	process(clk, rst)
	begin
		if rst= '1' then 
			idx <= 0;
		elsif rising_edge(clk) then
			if idx= 3 then idx<= 0; else idx<= idx + 1; end if;
		end if;
	end process;

	------------------------------------
	-- lcd signals
	
	process(clk, rst)
	begin
		if rst= '1' then
			cs    <= '0';
			ext_a <= (others=> '0');
			ext_d <= (others=> '0');
		elsif rising_edge(clk) then
			case idx is
				when 0=> ext_a<= "00" & rs;					-- set register adress
				when 1=> cs   <= '1';ext_d<= cmd;			-- write data, activate chip select
				when 2=> cs   <= '0';						-- deactivate chip select 
				when 3=> null;								-- data hold time
			end case;
		end if;
	end process;

	ext_rw<= '0';											-- always write
	cs_lcd <= cs;											-- assign external signals
	csn_lcd<= '0';											-- Data direction always -> LCD

	
	
	process(clk, rst)
	-- write ram data to LCD
	begin
		if rst= '1' then
			cmd	<= x"00";
			rs		<= '1';
			reset	<= '1';									-- activate LCD init at reset
			ram_adr<= "00000";
			cnt <= (others => '0');
		elsif rising_edge(clk) then
			if (reset ='0') then							-- if not LCD init						
				if idx = 2 then	cnt <= cnt + 1;
				end if;
				case cnt is
					when "000000" => ram_adr <= "00000"; cmd <= x"02";    rs <= '0';	-- cursor home
					when "000001" => ram_adr <= "00000"; cmd <= ram_data; rs <= '1';
					when "000010" => ram_adr <= "00001"; cmd <= ram_data; rs <= '1';
					when "000011" => ram_adr <= "00010"; cmd <= ram_data; rs <= '1';
					when "000100" => ram_adr <= "00011"; cmd <= ram_data; rs <= '1';
					when "000101" => ram_adr <= "00100"; cmd <= ram_data; rs <= '1';
					when "000110" => ram_adr <= "00101"; cmd <= ram_data; rs <= '1';
					when "000111" => ram_adr <= "00110"; cmd <= ram_data; rs <= '1';
					when "001000" => ram_adr <= "00111"; cmd <= ram_data; rs <= '1';
					when "001001" => ram_adr <= "01000"; cmd <= ram_data; rs <= '1';
					when "001010" => ram_adr <= "01001"; cmd <= ram_data; rs <= '1';
					when "001011" => ram_adr <= "01010"; cmd <= ram_data; rs <= '1';
					when "001100" => ram_adr <= "01011"; cmd <= ram_data; rs <= '1';
					when "001101" => ram_adr <= "01100"; cmd <= ram_data; rs <= '1';
					when "001110" => ram_adr <= "01101"; cmd <= ram_data; rs <= '1';
					when "001111" => ram_adr <= "01110"; cmd <= ram_data; rs <= '1';
					when "010000" => ram_adr <= "01111"; cmd <= ram_data; rs <= '1';
					when "010001" => ram_adr <= "00000"; cmd <= x"C0";    rs <= '0';	-- second line
					when "010010" => ram_adr <= "10000"; cmd <= ram_data; rs <= '1';
					when "010011" => ram_adr <= "10001"; cmd <= ram_data; rs <= '1';
					when "010100" => ram_adr <= "10010"; cmd <= ram_data; rs <= '1';
					when "010101" => ram_adr <= "10011"; cmd <= ram_data; rs <= '1';
					when "010110" => ram_adr <= "10100"; cmd <= ram_data; rs <= '1';
					when "010111" => ram_adr <= "10101"; cmd <= ram_data; rs <= '1';
					when "011000" => ram_adr <= "10110"; cmd <= ram_data; rs <= '1';
					when "011001" => ram_adr <= "10111"; cmd <= ram_data; rs <= '1';
					when "011010" => ram_adr <= "11000"; cmd <= ram_data; rs <= '1';
					when "011011" => ram_adr <= "11001"; cmd <= ram_data; rs <= '1';
					when "011100" => ram_adr <= "11010"; cmd <= ram_data; rs <= '1';
					when "011101" => ram_adr <= "11011"; cmd <= ram_data; rs <= '1';
					when "011110" => ram_adr <= "11100"; cmd <= ram_data; rs <= '1';
					when "011111" => ram_adr <= "11101"; cmd <= ram_data; rs <= '1';
					when "100000" => ram_adr <= "11110"; cmd <= ram_data; rs <= '1';
					when "100001" => ram_adr <= "11111"; cmd <= ram_data; rs <= '1';
					when others   => ram_adr <= "00000"; cmd <= x"02"; rs <= '0';cnt <= "000000";
				end case;
			else												-- if LCD init
				if idx = 2 then	cnt <= cnt + 1;
				end if;
				case cnt is
					when "000000" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
																-- wait (see LCD Datasheet for details)
					when "100100" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					when "101000" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					when "101100" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					when "110000" => cmd <= x"0c"; rs <= '0';	-- Display On/Off (display on, cursor off, cursor blink off)
					when "110100" => cmd <= x"06"; rs <= '0';	-- Entry Mode Set (increment on, display shift off)
					when "111000" => cmd <= x"01"; rs <= '0';	-- Display Clear
					when "111100" => cmd <= x"02"; rs <= '0';	-- Display Cursor Home
					when "111111" => reset <= '0'; cnt <= "000000";
					when others   => null;
				end case;
			end if;		
		end if;
	end process;

end bhv;
--------------------------------------------------------------------------------
-- end of file

