--------------------------------------------------------------------------------
-- Copyright (c) 2001 by Trenz Electronic GmbH.
-- Brendel 20, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2Se Spartan-IIe Development System.
-- File:        vga.vhd
-- Description: creates VGA Test pattern for TE-XC2Se.
-- History:     2002may8, TT, converted from te-bl.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

entity vga is
	port(
		CLK25:  in  STD_LOGIC;                     -- 25MHz clock (VGA)
		----------------------------               --
		vsync:  out STD_LOGIC;                     -- /vsync
		hsync:  out STD_LOGIC;                     -- /hsync
		blue:   out STD_LOGIC;                     -- blue
		green:  out STD_LOGIC;                     -- green
		red:    out STD_LOGIC;                     -- red
		bblue:  out STD_LOGIC;                     -- bright blue
		bgreen: out STD_LOGIC;                     -- bright green
		bred:   out STD_LOGIC                      -- bright red
		);
end vga;

--------------------------------------------------------------------------------
architecture bhv of vga is
	alias osc    :  STD_LOGIC is CLK25;  

	signal vr:      STD_LOGIC;                     -- VGA red
	signal vg:      STD_LOGIC;                     -- VGA green
	signal vb:      STD_LOGIC;                     -- VGA blue
	signal vbr:     STD_LOGIC;                     -- VGA intensity
	signal vbg:     STD_LOGIC;                     -- VGA intensity
	signal vbb:     STD_LOGIC;                     -- VGA intensity
	signal vcol:    STD_LOGIC_VECTOR(9 downto 0);  -- VGA column
	signal vrow:    STD_LOGIC_VECTOR(9 downto 0);  -- VGA row
	signal vvblk:   STD_LOGIC;                     -- VGA vertical blanking
	signal vhblk:   STD_LOGIC;                     -- VGA horizontal blanking

begin

	----------------------------------------------------------------------------
	-- vga output
	vga_blk: block
		signal   vclken:                 STD_LOGIC;
		
		signal   r_ii, g_ii,    b_ii:    STD_LOGIC; -- pre-blanking rgb
		signal   blk,  hblk_ii, vblk_ii: STD_LOGIC; -- blanking signals
		
		signal   hcnt:   UNSIGNED(vcol'range);
		signal   vcnt:   UNSIGNED(vrow'range);
		
		--        |__________a     d___          |___________e    h___
		-- rgb   _/__________\_____/___   lines _////////////_____////
		--        |                              |
		--       _|____________b c_____         _|____________f g_____
		-- hsync  |            \_/        vsync  |            \_/
		--
		--        d   = 31.77us                  h   = 16.6ms
		--        c-b =  3.77us                  g-f = 64us
		--        d-c =  1.89us                  h-g =  1.02ms
		--        a   = 25.17us                  e   = 15.25ms
		--        b-a =  0.94us                  f-e =  0.35ms
		-- 
		-- Mode       clock       h         v      a    b    c    d      e   f   g   h
		-- 640x480    25.175MHz   31.5kHz   60Hz   640  664  760  800    480 491 493 525
		-- 640x480    24MHz       31.5kHz   60Hz   640  645  730  762    480 491 493 525 (c-b=90, d-c=45)
		-- 800x600    36MHz       35.1kHz   --     800  824  896  1024   600 601 603 625
		-- 1024x768   65MHz       48.4kHz   --     1024 1032 1176 1344   768 771 777 806


		-- VESA Timing 25MHz Pixelclock
		--d = 31.77us = 794
		--a = 25.17us = 629 => *** 640 ***
		--b - a = 0.94us = 24 => b = 653
		--d - c = 1.89us = 47 => c = 747

		--for d=31.77us:
		--h = 16.6ms = 522
		--e = 15.25ms = *** 480 ***
		--f-e = 0.35ms = 11 => f = 491
		--h-g = 1.02ms = 32 => g = 490
		
		constant hcnt_a: UNSIGNED(hcnt'range):= TO_UNSIGNED(640, hcnt'length); -- blanking start
		constant hcnt_b: UNSIGNED(hcnt'range):= TO_UNSIGNED(653, hcnt'length); -- h-sync start
		constant hcnt_c: UNSIGNED(hcnt'range):= TO_UNSIGNED(747, hcnt'length); -- h-sync end
		constant hcnt_d: UNSIGNED(hcnt'range):= TO_UNSIGNED(794, hcnt'length); -- h length
		
		constant hcnt_v: UNSIGNED(hcnt'range):= TO_UNSIGNED(699, hcnt'length); -- next line
		
		constant vcnt_e: UNSIGNED(vcnt'range):= TO_UNSIGNED(480, vcnt'length); -- v blanking
		constant vcnt_f: UNSIGNED(vcnt'range):= TO_UNSIGNED(491, vcnt'length); -- v-sync start
		constant vcnt_g: UNSIGNED(vcnt'range):= TO_UNSIGNED(490, vcnt'length); -- v-sync end
		constant vcnt_h: UNSIGNED(vcnt'range):= TO_UNSIGNED(522, vcnt'length); -- v length
		
		begin
		
		------------------------------------
		-- i/o signals


		vr<= vcol(7);
		vg<= vcol(6);
		vb<= vcol(5);

		vbr<= vrow(7);
		vbg<= vrow(6);
		vbb<= vrow(5);
	
		red   <= vr and not(blk);
		green <= vg and not(blk);
		blue  <= vb and not(blk);

		bred  <= vbr and not(blk);
		bgreen <= vbg and not(blk);
		bblue  <= vbb and not(blk);
		
		vcol<= STD_LOGIC_VECTOR(hcnt);
		vrow<= STD_LOGIC_VECTOR(vcnt);
		
		blk  <= hblk_ii or vblk_ii;
		vhblk<= hblk_ii;
		vvblk<= vblk_ii;
		
		vclken<= '1';
		
		------------------------------------
		-- horizontal timing
		process(osc)
		begin
			if rising_edge(osc) then
				if vclken= '1' then
					if hcnt= hcnt_d then 
						hcnt   <= TO_UNSIGNED(0, hcnt'length);
						hblk_ii<= '0';
					else
						hcnt<= hcnt + 1; 
					end if;
					
					if hcnt= hcnt_a  then hblk_ii<= '1'; end if; -- blanking on
					if hcnt= hcnt_b  then hsync  <= '0'; end if; -- h-sync start
					if hcnt= hcnt_c  then hsync  <= '1'; end if; -- h-sync end
				end if;
				
				-- synopsys translate_off
				if hcnt(0)= 'U' then hcnt   <= TO_UNSIGNED(0, hcnt'length); end if;
				if hblk_ii= 'U' then hblk_ii<= '0';                         end if;
				if hsync= 'U'   then hsync  <= '1';                         end if;
				-- synopsys translate_on
			end if;
		end process;
		
		------------------------------------
		-- vertical timing
		process(osc)
		begin
			if rising_edge(osc) then
				if vclken= '1' then
					if hcnt= hcnt_v then
						if vcnt= vcnt_h then
							vcnt<= TO_UNSIGNED(0, vcnt'length);
							vblk_ii<= '0';
						else
							vcnt<= vcnt + 1;
						end if;
					end if;
					
					if vcnt= vcnt_e then vblk_ii<= '1'; end if; -- blanking on
					if vcnt= vcnt_f then vsync  <= '0'; end if; -- v-sync start
					if vcnt= vcnt_g then vsync  <= '1'; end if; -- v-sync end
				end if;
				
				-- synopsys translate_off
				if vcnt(0)= 'U' then vcnt   <= TO_UNSIGNED(0, vcnt'length); end if;
				if vblk_ii= 'U' then vblk_ii<= '0';                         end if;
				if vsync= 'U'   then vsync  <= '1';                         end if;
				-- synopsys translate_on
			end if;
		end process;
	end block;
	
end architecture;

--------------------------------------------------------------------------------
-- end of file
